#!/bin/perl -w
#################################################################################
# Created       : Martin Foster
# Modified      : 06/25/2001
#################################################################################
#
# Webchat Command line - Command line controls  part of Webchat
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5Th Street North-East
#                 Calgary, Alberta
#                 T2R 3W9
#
#################################################################################
use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Home;						# Home page handler

#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $func = shift @ARGV;						# Function command
my $type = shift @ARGV;						# Type to expire

#################################################################################
# Program Area
#################################################################################

	# Only continue if parameters are sent
	if (defined($func))
	{
		# Connect to Database
		Webchat::Database::DatabaseConnect(\$Database);

		# List audit entries
		if ($func eq '-audit')
		{ ListAudit($Database); }

		# List crypt
		if ($func eq '-crypt')
		{ ListCrypt($Database); }

		# Listing of E-Mails
		if ($func eq '-list')
		{ ListEmail($Database); }

		# Listing of E-Mails
		if ($func eq '-summary')
		{ ListSummary($Database); }
	
		# Expiery control
		if ($func eq '-expire')
		{
			# Specific expire
			if (defined($type))
			{
				# Bannings
				if ($type eq 'ban')
				{ ExpireBans($Database); }

				# Bannings
				if ($type eq 'mail')
				{ ExpireMail($Database); }

				# Posts
				elsif ($type eq 'post')
				{ ExpirePosts($Database); }

				# Puppeteers
				elsif ($type eq 'puppeteer')
				{ ExpirePuppeteer($Database); }

				# Puppets
				elsif ($type eq 'puppet')
				{ ExpirePuppet($Database); }

				# Private realms
				elsif ($type eq 'realm')
				{ ExpireRealm($Database); }

				# Registrations
				elsif ($type eq 'registration')
				{ ExpireRegistrations($Database); }
			}

			# General expire
			else
			{
				# Run though all methods
				ExpireBans($Database);
				ExpireMail($Database);
				ExpirePosts($Database);
				ExpirePuppeteer($Database);
				ExpirePuppet($Database);
				ExpireRealm($Database);
				ExpireRegistrations($Database);
			}
		}
		$Database->disconnect();			# Disconnect database handle
	}

	# Display help
	unless (defined($func))
	{
		print "Webchat Command-Line Interface:\n";
		print "Usage: webchatcmd.plx {-list|-expire} [expire]\n\n";
		print "-audit\n";
		print "\tDisplay contents of audit table\n\n";
		print "-crypt\n";
		print "\tProduces output for Apache password files\n\n";
		print "-list\n";
		print "\tDisplay a list of all E-Mail addresses of members\n\n";
		print "-summary\n";
		print "\tSummary of room useage\n\n";
		print "-expire\n";
		print "\tWill remove expired entires.  When called alone all are removed\n";
		print "\tor if specified only a specific task will take place as listed\n";
		print "\tbelow:\n";
		print "\t\tban          - Remove old bannings\n";
		print "\t\tmail         - Remove old mail\n";
		print "\t\tpost         - Remove old posts\n";
		print "\t\tpuppet       - Remove inactive puppets\n";
		print "\t\tpuppeteer    - Remove inactive puppeteers\n";
		print "\t\trealm        - Remove inactive private realms\n";
		print "\t\tregistration - Remove old incomplete registrations\n\n";
	}

#################################################################################
# Sub-Routines
#################################################################################

####################
# List Audit
#
# Produces an output, which can be used for a text file containing all of the 
# audit entires in the database itself.   This will of course work from lastest
# to earliest entries.

sub ListAudit
{
	####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Database statement

	my $audit;						# Action completed


	####################
	# Program area

	# Prepare and execute
	$Statement = $Database->prepare("SELECT 
			  AuditTimestamp,
			  PuppeteerLogin,
			  PuppeteerHostname,
			  AuditAction
			FROM Audit"); 
	$Statement->execute();

	# Audit trail
	while ($audit = $Statement->fetchrow_hashref())
	{
		print "$audit->{AuditTimestamp} ($audit->{PuppeteerLogin} \@ $audit->{PuppeteerHostname}) - $audit->{AuditAction}\n";
	}

	# Finsih call
	$Statement->finish();
}

####################
# List Crypt
#
# Lists the encrypted usernames and passwords for supervisors and the
# administrators to allow for building password files for Apache.

sub ListCrypt
{
	####################
	# Data Members
	my $Database = shift;					# Database handle
	my $Statement;						# Database statement
	
	my $crypt;						# Action completed

	
	####################
	# Program area

	# Prepare and execute
	$Statement = $Database->prepare("SELECT 
			  PuppeteerLogin,
			  PuppeteerPassword
			FROM  Puppeteer
			WHERE (PuppeteerAdminPrivs='yes'
			   OR  PuppeteerSuperPrivs='yes')"); 
	$Statement->execute();

	# Crypted Password
	while ($crypt = $Statement->fetchrow_hashref())
	{
		print "$crypt->{PuppeteerLogin}:$crypt->{PuppeteerPassword}\n";
	}

	# Finsih call
	$Statement->finish();
}

####################
# List EMail
#
# Is used to simply list of all E-Mail addresses of those who are members of the
# chat server.  This is useful to embed within Majordomo or other such mailing
# list system.

sub ListEmail							# Database
{
	####################
	# Data members
	my $Database = shift;					# Database handle

	my $puppeteers;						# Single puppeteer EMail
	my @puppeteers;						# List of all EMails


	####################
	# Program area

	# Retreive puppeteers E_Mail addresses
	Webchat::Database::GetListEMails($Database, \@puppeteers);

	# Print each E-Mail address
	foreach $puppeteers (@puppeteers)
	{ print "$puppeteers\n"; }
}


####################
# List Summary
#
# Somewhat inneficient function that will display a summary all the useage
# in all the rooms.   Public or otherwise.
#


sub ListSummary							# Database
{
	####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement handle

	my $number;						# Number
	my $name;						# Name of room

	my $users;						# Amount of users
	my $puppets;						# Amount of puppets

	my $day;						# Users in day
	my $week;						# Users in week
	my $month;						# Users in month
	my $expire;						# About to expire
	my $elapsed;						# Calculation of elapsed time

	my %hits;						# Total hits
	my %posts;						# Total posts


	####################
	# Program area



	# Prepare and execute
	# Accurate posting
	$Statement = $Database->prepare("SELECT COUNT(*), 
				  RealmName 
				FROM  Post 
				WHERE PostTo <> 'system' 
				GROUP BY RealmName
				ORDER BY RealmName");
	$Statement->execute();

	while (($number, $name) = $Statement->fetchrow())
	{ $posts{$name} = $number };

	# Finish
	$Statement->finish();

	# Total posts
	$Statement = $Database->prepare("SELECT COUNT(*), 
				  RealmName 
				FROM  Post 
				GROUP BY RealmName
				ORDER BY RealmName");
	$Statement->execute();

	while (($number, $name) = $Statement->fetchrow())
	{ $hits{$name} = $number };

	# Finish
	$Statement->finish();


	# Room useage
	# Initial display
	print "Posts   Hits    Realm Name\n";
	print "------- ------- ----------------------------------\n";

	# Retreive and display
	foreach $name (sort(keys(%posts)))
	{ print "$posts{$name}\t$hits{$name}\t$name\n"; }


	# Chat Preferences
	# Initial Display
	print "\n\nHits    Chat Type\n";
	print "------- ----------------------------------\n";

	# Prepare and execute
	$Statement = $Database->prepare("SELECT COUNT(*), PuppeteerChatPref FROM Puppeteer GROUP BY PuppeteerChatPref");
	$Statement->execute();

	# Retreive and display
	while (($number, $name) = $Statement->fetchrow())
	{ print "$number\t$name\n"; }

	# Finish
	$Statement->finish();


	# Puppet Useage
	# Initial display
	print "\n\nFreqency Count   Percent\n";
	print "-------- ------- -------\n";


	# Totals
	($users)   =  Webchat::Database::DataGet($Database, 'SELECT COUNT(*) FROM Puppeteer');
	($puppets) =  Webchat::Database::DataGet($Database, 'SELECT COUNT(*) FROM Puppet');

	# Day use
	$elapsed   = time - 86400; 
	($day)     = Webchat::Database::DataGetFrenquency($Database, $elapsed);
	printf "Day      %7i   %5.2f\n", $day, (($day/$users)*100);	

	# Weekly use
	$elapsed   = time - (86400*7);
	($week)    = Webchat::Database::DataGetFrenquency($Database, $elapsed);
	printf "Week     %7i   %5.2f\n", $week, (($week/$users)*100);	

	# Monthly use
	$elapsed   = time - (86400*30);
	($month)   = Webchat::Database::DataGetFrenquency($Database, $elapsed);
	printf "Month    %7i   %5.2f\n", $month, (($month/$users)*100);	

	# About to expire
	$elapsed   = time - (86400*89);
	($expire)  = Webchat::Database::DataGet($Database, "SELECT COUNT(*) FROM Puppeteer WHERE PuppeteerTimestamp < $elapsed");
	printf "Expire   %7i   %5.2f\n", $expire, (($expire/$users)*100);	

	print "\n\nTotals\n-------\nPuppeteers: $users\nPuppets: $puppets\n";
}


####################
# Expire Bans
# 
# Will expire all bans older then a certain time.

sub ExpireBans
{
	####################
	# Data members
	my $Database = shift;					# Database handle
	my $time;						# Stale time

	my %system;						# System hash


	####################
	# Program area

	# Calculate time
	Webchat::Database::GetHashSystem($Database, \%system);
	$time = (time - ($system{'SetTimeoutBanned'} * $system{'SetTimeDay'})); 

	# Remove old posts
	$Database->do("DELETE FROM PuppeteerRestriction WHERE RestrictionTimestamp < ?", {}, $time);
}

####################
# Expire mail
# 
# Will expire all mail older then a certain time.

sub ExpireMail
{
	####################
	# Data members
	my $Database = shift;					# Database handle
	my $time;						# Stale time

	my %system;						# System hash


	####################
	# Program area

	# Calculate time
	Webchat::Database::GetHashSystem($Database, \%system);
	$time = (time - ($system{'SetTimeoutMail'} * $system{'SetTimeDay'})); 

	# Remove old posts
	$Database->do("DELETE FROM Mail WHERE MailTimestamp < ?", {}, $time);
}


####################
# Expire Posts
# 
# Will expire all posts which are older then twenty minutes.

sub ExpirePosts
{
	####################
	# Data members
	my $Database = shift;					# Database handle
	my $time;						# Stale time

	####################
	# Program area

	# Calculate time
	$time = (time - 1200); 

	# Remove old posts
	$Database->do("DELETE FROM Post WHERE PostTimestamp < ?", {}, $time);
}


####################
# Expire Puppets
# 
# Expires inactive puppets this will occur at different intervals then the 
# expiration of the puppeteer

sub ExpirePuppet
{
	####################
	# Data members
	my $Database = shift;					# Database handle
	my $time;						# Stale time

	my %system;						# System hash


	####################
	# Program area

	# Calculate time for which older are destroyed
	Webchat::Database::GetHashSystem($Database, \%system);
	$time = (time - ($system{'SetTimeoutPuppet'} * $system{'SetTimeDay'}));

	# Remove old posts
	$Database->do("DELETE FROM Puppet 
		WHERE PuppetActive IS NOT NULL
		  AND PuppetActive < ?", {},  $time);
}


####################
# Expire Puppeteer
#
# Will remove all puppeteers that have lapsed on their membership times.  Which
# is determine in the system DBM.

sub ExpirePuppeteer
{
	####################
	# Data members
	my $Database = shift;					# Database handle

	my $home;						# Home page
	my $puppeteer;						# Individual puppeteer
	my $puppet;						# Individual puppet
	my $time;						# Time in seconds
	
	my @puppeteers;						# List of puppeteers
	my @puppets;						# List of puppets

	my %home;						# Home page hash
	my %system;						# System hash


	####################
	# Program area
	
	# Link with hash
	Webchat::Database::GetHashHome($Database, \%home);
	Webchat::Database::GetHashSystem($Database, \%system);

	# Calculate time for which older are destroyed
	$time = (time - ($system{'SetTimeoutPuppeteer'} * $system{'SetTimeDay'})); 

	# Rerieve list of puppeteers
	Webchat::Database::GetListExpire($Database, \@puppeteers, $time);

	# Run though list of puppeteers	
	foreach $puppeteer (@puppeteers)
	{
		# Home page handling
		# Retreive home
		($home) = Webchat::Database::DataGetPuppeteerHome($Database, $puppeteer);

		# Remove folder
		if (defined($home))
	 	{
			# Attempt to remove, else error 
			unless (Webchat::Home::DirClear("$home{SetRoot}/$home")) 
			{ print "\nERROR: Home directory $home{SetRoot}/$home not cleared ($!)\n";}
		}


		# Database handling
		# Retreive list of puppets 
		Webchat::Database::GetListPuppetYours($Database, \@puppets, $puppeteer);

		# Run though list of puppets
		while ($puppet = shift @puppets)
		{
			# Remove instance of puppets
			$Database->do("DELETE FROM PuppetInvitation WHERE PuppetName=?", {}, $puppet);
			$Database->do("DELETE FROM PuppetIgnore     WHERE PuppetName=?", {}, $puppet);
			$Database->do("DELETE FROM Puppet           WHERE PuppetName=?", {}, $puppet);
		}

		# Remove all instances of the puppeteers (except from Audit)
		$Database->do("DELETE FROM Mail                  WHERE PuppeteerLogin=?", {}, $puppeteer);
		$Database->do("DELETE FROM RealmSupervisor       WHERE PuppeteerLogin=?", {}, $puppeteer);
		$Database->do("DELETE FROM PuppeteerRestriction  WHERE PuppeteerLogin=?", {}, $puppeteer);
		$Database->do("DELETE FROM PuppeteerRegistration WHERE PuppeteerLogin=?", {}, $puppeteer);
		$Database->do("DELETE FROM PuppetIgnore          WHERE PuppeteerLogin=?", {}, $puppeteer);
		$Database->do("DELETE FROM Puppeteer             WHERE PuppeteerLogin=?", {}, $puppeteer);
	}
}


####################
# Expire Realm
#
# Will remove all private realms after elapsed time.

sub ExpireRealm
{
	####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# SQL Statement

	my $time;						# Time in seconds
	my $realm;						# Individual puppeteer
	
	my %system;						# System hash


	####################
	# Program area

	# Calculate time for which older are destroyed
	Webchat::Database::GetHashSystem($Database, \%system);
	$time = (time - ($system{'SetTimeoutRealm'} * $system{'SetTimeDay'})); 
	
	# Prepare and execute
	$Statement = $Database->prepare("SELECT RealmName FROM Realm WHERE RealmPublic='no' AND RealmTimestamp < ?");
	$Statement->execute($time);

	# Run though list
	while ($realm = $Statement->fetchrow())
	{
		# Remove affected entries
		$Database->do("DELETE FROM Post                 WHERE RealmName=?", {}, $realm);
		$Database->do("DELETE FROM WordRestriction      WHERE RealmName=?", {}, $realm);
		$Database->do("DELETE FROM RealmSupervisor      WHERE RealmName=?", {}, $realm);
		$Database->do("DELETE FROM PuppetInvitation     WHERE RealmName=?", {}, $realm);
		$Database->do("DELETE FROM PuppeteerRestriction WHERE RealmName=?", {}, $realm);		
		$Database->do("DELETE FROM Realm                WHERE RealmName=?", {}, $realm);
	}
	$Statement->finish();					# End query
}


####################
# Expire Registrations
# 
# Will expire all registration older then a certain taime who have not completed
# their registration.

sub ExpireRegistrations
{
	####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Sql Statement

	my $time;						# Stale time
	my $puppeteer;						# Individual puppeteer

	my %system;						# System hash


	####################
	# Program area

	# Calculate time
	Webchat::Database::GetHashSystem($Database, \%system);
	$time = (time - ($system{'SetTimeoutRegister'} * $system{'SetTimeDay'})); 


	# Initialize and execute
	$Statement = $Database->prepare("SELECT PuppeteerLogin FROM PuppeteerRegistration WHERE PuppeteerTimeStamp < ?");		
	$Statement->execute($time);

	# Cycle though list
	while ($puppeteer = $Statement->fetchrow())
	{
		# Remove unregistered puppeteers
		$Database->do("DELETE FROM Puppeteer             WHERE PuppeteerLogin=?", {}, $puppeteer);
		$Database->do("DELETE FROM PuppeteerRegistration WHERE PuppeteerLogin=?", {}, $puppeteer);
	}
	$Statement->finish();					# End query
}
